searchData={"items":[{"type":"module","title":"HTTPEx","doc":"HTTPEx provides common functionality to do HTTP calls like GET and POST via a HTTP client (like HTTPoison, Finch etc.).\n\nIt adds a couple of additional features on top of a HTTP library:\n- Adds OT tracing\n- Adds a standarized Request, Response and Error struct\n- Switch between different HTTP clients (like HTTPoison, Finch etc.)\n- Adds a standarized response\n- Adds retry mechanism when timeout of connection closed is encountered\n- Adds a stub HTTP client that is able to mock requests for all possibile implementations that use this module\n\nSee HTTPEx.TestBackend to see how stubbing out requests and responses work.","ref":"HTTPEx.html"},{"type":"function","title":"HTTPEx.get/2","doc":"Executes GET request","ref":"HTTPEx.html#get/2"},{"type":"function","title":"Examples - HTTPEx.get/2","doc":"iex> HTTPEx.get(\"http://www.example.com\", backend: MockBackend)\n    {:ok, %HTTPEx.Response{body: \"OK!\", client: :httpoison, retries: 1, status: 200, parsed_body: nil, headers: []}}\n\n    iex> HTTPEx.get(\"http://www.example.com\", backend: MockBackend, client: :finch)\n    {:ok, %HTTPEx.Response{body: \"OK!\", client: :httpoison, retries: 1, status: 200, parsed_body: nil, headers: []}}\n\n    iex> HTTPEx.get(\"http://www.example.com/json\", headers: [{\"Content-Type\", \"application/json\"}], backend: MockBackend)\n    {\n      :ok,\n      %HTTPEx.Response{\n        body: \"{\\\"payload\\\":{\\\"items\\\":[1,2,3]}}\",\n        client: :httpoison,\n        headers: [],\n        parsed_body: %{\"payload\" => %{\"items\" => [1, 2, 3]}},\n        retries: 1,\n        status: 202\n      }\n    }\n\n    iex> HTTPEx.get(\"http://www.example.com/error\", backend: MockBackend)\n    {\n      :error,\n      %HTTPEx.Error{\n        body: \"{\\\"errors\\\":[{\\\"code\\\":\\\"invalid_payload\\\"}]}\",\n        client: :httpoison,\n        headers: [],\n        parsed_body: %{\"errors\" => [%{\"code\" => \"invalid_payload\"}]},\n        retries: 1,\n        status: 422,\n        reason: :unprocessable_entity\n      }\n    }","ref":"HTTPEx.html#get/2-examples"},{"type":"function","title":"HTTPEx.get_header/2","doc":"Gets value of request, response or error headers.","ref":"HTTPEx.html#get_header/2"},{"type":"function","title":"Examples - HTTPEx.get_header/2","doc":"iex> HTTPEx.get_header(%Request{method: :get, url: \"http://example.com\", headers: [{\"Content-Type\", \"application/json\"}, {\"Secret\", \"123\"}]}, \"content-type\")\n  \"application/json\"\n\n  iex> HTTPEx.get_header(%Response{headers: [{\"Content-Type\", \"application/json\"}, {\"Secret\", \"123\"}]}, \"Secret\")\n  \"123\"\n\n  iex> HTTPEx.get_header(%Response{headers: [{\"Content-Type\", \"application/json\"}, {\"Secret\", \"123\"}]}, \"non-existent-key\")\n  nil","ref":"HTTPEx.html#get_header/2-examples"},{"type":"function","title":"HTTPEx.post/3","doc":"Executes POST request","ref":"HTTPEx.html#post/3"},{"type":"function","title":"Examples - HTTPEx.post/3","doc":"iex> HTTPEx.post(\"http://www.example.com\", JSON.encode!(%{\"data\" => true}), backend: MockBackend)\n    {:ok, %HTTPEx.Response{body: \"OK!\", client: :httpoison, retries: 1, status: 200, parsed_body: nil, headers: []}}","ref":"HTTPEx.html#post/3-examples"},{"type":"function","title":"HTTPEx.request/1","doc":"Executes a Request","ref":"HTTPEx.html#request/1"},{"type":"function","title":"Examples - HTTPEx.request/1","doc":"iex> request = %HTTPEx.Request{\n    ...>   method: :get,\n    ...>   client: :finch,\n    ...>   url: \"http://www.example.com\",\n    ...>   options: [backend: MockBackend],\n    ...> }\n    ...> HTTPEx.request(request)\n    {:ok, %HTTPEx.Response{body: \"OK!\", client: :httpoison, retries: 1, status: 200, parsed_body: nil, headers: []}}\n\n    iex> request = %HTTPEx.Request{\n    ...>   method: :post,\n    ...>   body: JSON.encode!(%{\"data\" => true}),\n    ...>   url: \"http://www.example.com/json\",\n    ...>   headers:  [{\"Content-Type\", \"application/json\"}],\n    ...>   options: [backend: MockBackend],\n    ...> }\n    ...> HTTPEx.request(request)\n    {\n      :ok,\n      %HTTPEx.Response{\n        body: \"{\\\"payload\\\":{\\\"label\\\":\\\"ABCD\\\"}}\",\n        client: :httpoison,\n        headers: [],\n        parsed_body: %{\"payload\" => %{\"label\" => \"ABCD\"}},\n        retries: 1,\n        status: 200\n      }\n    }","ref":"HTTPEx.html#request/1-examples"},{"type":"function","title":"HTTPEx.to_response/1","doc":"","ref":"HTTPEx.html#to_response/1"},{"type":"function","title":"HTTPEx.to_response/2","doc":"","ref":"HTTPEx.html#to_response/2"},{"type":"module","title":"HTTPEx.Backend.Default","doc":"The default backend for HTTP calls.\nUses actual the actual http implementation.","ref":"HTTPEx.Backend.Default.html"},{"type":"function","title":"HTTPEx.Backend.Default.request/1","doc":"","ref":"HTTPEx.Backend.Default.html#request/1"},{"type":"function","title":"HTTPEx.Backend.Default.request_options/1","doc":"","ref":"HTTPEx.Backend.Default.html#request_options/1"},{"type":"module","title":"HTTPEx.Backend.Mock","doc":"Backend for mocked HTTP requests","ref":"HTTPEx.Backend.Mock.html"},{"type":"module","title":"Stubbing requests - HTTPEx.Backend.Mock","doc":"To add a stubbed request mock, use `stub_request!/1`.\nThis means that the request _may_ be made but is not asserted.\n\nThis is usually done in the beginning of your test suite.\nThese stubs are global and are not isolated. Every test, process etc. will be able to access these.","ref":"HTTPEx.Backend.Mock.html#module-stubbing-requests"},{"type":"module","title":"Asserted requests - HTTPEx.Backend.Mock","doc":"To add an assertion, use `expect_request!/1`.\nThis means that the test will fail, if the HTTP request is not made.\nThis is usally done on a case per case basis in your tests.\n\nThese assertions are isolated and are only accessible for the test you are currently running.\nProcesses that were spawned while the test are running, will also be able to access those defined assertions.","ref":"HTTPEx.Backend.Mock.html#module-asserted-requests"},{"type":"module","title":"Matching and expectations inside defined stubs or assertions - HTTPEx.Backend.Mock","doc":"There is a difference between a _match_ and an _expectation_ within a configured mock or expectation.\nGiven the following example:\n\n```elixir\nexpect_request!(\n  host: \"localhost\",\n  port: 3000,\n  response: %{\n    status: 200,\n    body: \"OK\"\n  }\n)\n```\n\nThis adds a match on `host` and `port`. So every request made to `localhost:3000`.\n\nWhen a HTTP call is made, the Mock library will try to look through the registered stubs and asserts\nand will try find a request that matches the value provided in `host`.\n\nHowever, there is also the possibility to assert the actually made request.\nThis is handy when you want to explicitly verify that the request you are making, is using the right\npayload, headers etc.\n\n```elixir\nexpect_request!(\n  endpoint: \"http://localhost:3000/api/rates\",\n  expect_body: JSON.encode!(%{\"user_id\" => \"1337\"}),\n  expect_body_format: :json\n)\n```\n\nIn this case, the Mock library will match all requests made to port 3000, host localhost,\npath `api/rates`. When that request is made, it will verify that the used `body` matches the one\nin `expect_body`. When that is not the case, the test will raise an `AssertionError`.","ref":"HTTPEx.Backend.Mock.html#module-matching-and-expectations-inside-defined-stubs-or-assertions"},{"type":"function","title":"HTTPEx.Backend.Mock.assert_no_request!/1","doc":"The inverse of an expected call. Checks that the call (using the matchers) wasn't made.","ref":"HTTPEx.Backend.Mock.html#assert_no_request!/1"},{"type":"function","title":"Examples - HTTPEx.Backend.Mock.assert_no_request!/1","doc":"iex> HTTPEx.Mock.assert_no_request!(\n    ...>   method: :get,\n    ...>   endpoint: \"http://www.example.com\"\n    ...> )","ref":"HTTPEx.Backend.Mock.html#assert_no_request!/1-examples"},{"type":"function","title":"HTTPEx.Backend.Mock.expect_request!/1","doc":"Adds an asserted request. It will require a match always on `endpoint`, `method` and (optional) `caller`.\nIf you try to add an expected request that already exists, this function will throw an exception.\n\nIf you want to explicitly override the request, you must set the `allow_override` option.","ref":"HTTPEx.Backend.Mock.html#expect_request!/1"},{"type":"function","title":"Options - HTTPEx.Backend.Mock.expect_request!/1","doc":"* `body` -\n  The request body that is to be matched when an HTTP call is made.\n\n* `body_format` -\n  Sets the matching body format. By using this, the matcher can\n  do a safe comparison. This is handy if your fixtures are formatted\n  but your requests are not.\n\n  Can be one of:\n  - `json`\n  - `xml`\n  - `form`\n\n* `description` -\n  Described the expectation.\n\n* `expect_body` -\n  The request body that is expected to be used.\n\n* `expect_body_format` -\n  Sets the expected body format. By using this, the matcher can\n  do a safe comparison. This is handy if your fixtures are formatted\n  but your requests are not.\n\n  Can be one of:\n  - `json`\n  - `xml`\n  - `form`\n\n* `expect_headers` -\n  The request headers that are expected to be used.\n\n* `expect_path` -\n  The request path that is to be expected to be used.\n\n* `expect_query` -\n  The request url query that is to be expected to be used.\n\n* `headers` -\n  The request headers that are to be matched when an HTTP call is made.\n\n* `host` -\n  The request host that is to be matched when an HTTP call is made.\n  You can also use `endpoint` instead of this option.\n\n* `endpoint` -\n  The request URL that is to be matched when an HTTP call is made.\n\n* `min_calls` number() | nil (default 1) -\n  Sets the minimum number of calls that are allowed.\n  If you set this to `nil`, the request is not mandatory.\n\n* `max_calls` number() | nil (default nil) -\n  Sets the maximum number of the times the request can be made.\n\n* `method` atom() -\n  The method that is to be matched when an HTTP call is made.\n\n* `path` String.t() | function() -\n  The request path that is to be matched when an HTTP call is made.\n  You can also use `path` instead of this option.\n\n* `port` number() -\n  The request port that is to be matched when an HTTP call is made.\n  You can also use `endpoint` instead of this option.\n\n* `query` map() | function() -\n  The url query that is to be matched when an HTTP call is made.\n  You can also pass this down to `endpoint` or in combination with `endpoint`.\n\n* `response` {:ok, map() | HTTPoison.Response.t()} | {:error, atom() | HTTPoison.Error.t()} -\n  Sets the fake response. Can be one of:\n  * {:ok, %{status: _status_code, body: _body, headers: _headers}}\n  * {:ok, %HTTPoison.Response{}}\n  * {:error, :timeout}\n  * {:error, %HTTPoison.Error{}}","ref":"HTTPEx.Backend.Mock.html#expect_request!/1-options"},{"type":"function","title":"Examples - HTTPEx.Backend.Mock.expect_request!/1","doc":"iex> HTTPEx.Mock.expect_request!(\n    ...>   method: :get,\n    ...>   max_calls: 2,\n    ...>   endpoint: \"http://www.example.com\",\n    ...>   response: %{status: 200, body: \"OK!\", headers: [{\"Content-Type\", \"application/json\"}]},\n    ...> )","ref":"HTTPEx.Backend.Mock.html#expect_request!/1-examples"},{"type":"function","title":"HTTPEx.Backend.Mock.request/1","doc":"Returns a response from one of the expectations","ref":"HTTPEx.Backend.Mock.html#request/1"},{"type":"function","title":"HTTPEx.Backend.Mock.start/0","doc":"Starts the mock registries in a supervisor.\nWe have a server running for the global mocks and local mocks.","ref":"HTTPEx.Backend.Mock.html#start/0"},{"type":"function","title":"HTTPEx.Backend.Mock.start_assertions/0","doc":"Start the HTTPEx Mock server for local expectations.","ref":"HTTPEx.Backend.Mock.html#start_assertions/0"},{"type":"function","title":"HTTPEx.Backend.Mock.start_stubs/0","doc":"Start the HTTPEx Mock server for global expectations.","ref":"HTTPEx.Backend.Mock.html#start_stubs/0"},{"type":"function","title":"HTTPEx.Backend.Mock.stub_request!/1","doc":"Adds a stubbed HTTP request. Does not check if the request is actually made.\nThis is used to setup basic HTTP requests all your tests can use.","ref":"HTTPEx.Backend.Mock.html#stub_request!/1"},{"type":"function","title":"Examples - HTTPEx.Backend.Mock.stub_request!/1","doc":"iex> HTTPEx.Mock.stub_request!(\n    ...>   method: :get,\n    ...>   endpoint: \"http://www.example.com\",\n    ...>   response: %{status: 200, body: \"OK\"}\n    ...> )","ref":"HTTPEx.Backend.Mock.html#stub_request!/1-examples"},{"type":"function","title":"Options - HTTPEx.Backend.Mock.stub_request!/1","doc":"See `expect_request!/1`","ref":"HTTPEx.Backend.Mock.html#stub_request!/1-options"},{"type":"function","title":"HTTPEx.Backend.Mock.verify!/1","doc":"Manually verify made calls","ref":"HTTPEx.Backend.Mock.html#verify!/1"},{"type":"function","title":"HTTPEx.Backend.Mock.verify_on_exit!/0","doc":"Verifies made HTTP calls.\nAdds a ExUnit hook and checks if all HTTP calls were actually made.","ref":"HTTPEx.Backend.Mock.html#verify_on_exit!/0"},{"type":"module","title":"HTTPEx.Backend.Mock.Expectation","doc":"Defines a HTTP mock expectation.\nConsists of a matching part and an expectation part.","ref":"HTTPEx.Backend.Mock.Expectation.html"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.enum_matcher/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:enum_matcher/0"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.exact_value_matcher/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:exact_value_matcher/0"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.expects_result/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:expects_result/0"},{"type":"function","title":"HTTPEx.Backend.Mock.Expectation.find_matching_expectation/2","doc":"Finds the expectation that matches the given Request.t()","ref":"HTTPEx.Backend.Mock.Expectation.html#find_matching_expectation/2"},{"type":"function","title":"Example - HTTPEx.Backend.Mock.Expectation.find_matching_expectation/2","doc":"iex> expectation_1 =\n    ...>   %Expectation{index: 0}\n    ...>   |> Expectation.set_match!(:host, \"www.example.com\")\n    ...>   |> Expectation.set_match!(:port, 80)\n    ...>\n    ...> expectation_2 =\n    ...>   %Expectation{index: 1}\n    ...>   |> Expectation.set_match!(:host, \"www.example.com\")\n    ...>   |> Expectation.set_match!(:path, Regex.compile!(\"api/(? [^/]+)/path/*\"))\n    ...>   |> Expectation.set_match!(:port, 80)\n    ...>\n    ...> expectations = [expectation_1, expectation_2]\n    ...>\n    ...> {:ok, match, vars} =\n    ...>   Expectation.find_matching_expectation(\n    ...>     expectations,\n    ...>     %Request{\n    ...>       url: \"http://www.example.com/api/v1/path/test?token=XYZ&user_id=1337\",\n    ...>       body: \"Payload OK!\",\n    ...>       headers: [{\"app\", \"test\"}, {\"secret\", \"123\"}],\n    ...>       method: :post\n    ...>     }\n    ...>   )\n    ...>\n    ...> match == expectation_1\n    true\n    iex> vars\n    %{}\n    iex> {:ok, match, vars} =\n    ...>   Expectation.find_matching_expectation(\n    ...>     expectations,\n    ...>     %Request{\n    ...>       url: \"http://www.example.com/another-path\",\n    ...>       body: \"Payload OK!\",\n    ...>       headers: [{\"app\", \"test\"}, {\"secret\", \"123\"}],\n    ...>       method: :post\n    ...>     }\n    ...>   )\n    ...>\n    ...> match == expectation_1\n    true\n    iex> vars\n    %{}","ref":"HTTPEx.Backend.Mock.Expectation.html#find_matching_expectation/2-example"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.func_matcher/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:func_matcher/0"},{"type":"function","title":"HTTPEx.Backend.Mock.Expectation.get_expect/2","doc":"Gets expects field from given Expectation","ref":"HTTPEx.Backend.Mock.Expectation.html#get_expect/2"},{"type":"function","title":"Examples - HTTPEx.Backend.Mock.Expectation.get_expect/2","doc":"iex> expectation = Expectation.set_expect!(%Expectation{}, :body, \"OK!\")\n    iex> Expectation.get_expect(expectation, :body)\n    \"OK!\"\n    iex> Expectation.get_expect(expectation, :path)\n    :any","ref":"HTTPEx.Backend.Mock.Expectation.html#get_expect/2-examples"},{"type":"function","title":"HTTPEx.Backend.Mock.Expectation.get_match/2","doc":"Gets matcher field from given Expectation","ref":"HTTPEx.Backend.Mock.Expectation.html#get_match/2"},{"type":"function","title":"Examples - HTTPEx.Backend.Mock.Expectation.get_match/2","doc":"iex> expectation = Expectation.set_match!(%Expectation{}, :host, \"localhost\")\n    iex> Expectation.get_match(expectation, :host)\n    \"localhost\"\n    iex> Expectation.get_match(expectation, :headers)\n    :any\n\n    iex> expectation = Expectation.set_match!(%Expectation{}, :path, fn _request -> true end)\n    iex> is_function(Expectation.get_match(expectation, :path))\n    true","ref":"HTTPEx.Backend.Mock.Expectation.html#get_match/2-examples"},{"type":"function","title":"HTTPEx.Backend.Mock.Expectation.get_matcher_type/2","doc":"Looks up which type the given matcher field is","ref":"HTTPEx.Backend.Mock.Expectation.html#get_matcher_type/2"},{"type":"function","title":"Examples - HTTPEx.Backend.Mock.Expectation.get_matcher_type/2","doc":"iex> Expectation.get_matcher_type(:host, \"localhost\")\n    :string\n\n    iex> Expectation.get_matcher_type(:body, {\"{}\", :json})\n    :string_with_format\n\n    iex> Expectation.get_matcher_type(:body, {\" b \", :xml})\n    :string_with_format\n\n    iex> Expectation.get_matcher_type(:body, {\"foo=bar\", :form})\n    :string_with_format\n\n    iex> Expectation.get_matcher_type(:body, {:exact_value, {:form, \"data\"}})\n    :exact_value\n\n    iex> Expectation.get_matcher_type(:host, fn _request -> true end)\n    :func\n\n    iex> Expectation.get_matcher_type(:query, %{\"user_id\" => \"1234\"})\n    :map\n\n    iex> Expectation.get_matcher_type(:headers, [{\"Content-Type\", \"application/json\"}])\n    :keyword_list\n\n    iex> Expectation.get_matcher_type(:path, :any)\n    :wildcard\n\n    iex> Expectation.get_matcher_type(:port, 1337)\n    :int\n\n    iex> Expectation.get_matcher_type(:path, ~r/api/)\n    :regex\n\n    iex> Expectation.get_matcher_type(:method, :post)\n    :enum\n\n    iex> Expectation.get_matcher_type(:method, :get)\n    :enum","ref":"HTTPEx.Backend.Mock.Expectation.html#get_matcher_type/2-examples"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.http_response/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:http_response/0"},{"type":"function","title":"HTTPEx.Backend.Mock.Expectation.increase_call/1","doc":"Increases the calls counter in Expectation","ref":"HTTPEx.Backend.Mock.Expectation.html#increase_call/1"},{"type":"function","title":"Example - HTTPEx.Backend.Mock.Expectation.increase_call/1","doc":"iex> expectation = %Expectation{}\n    ...> expectation.calls\n    0\n    iex> expectation = Expectation.increase_call(expectation)\n    ...> expectation.calls\n    1","ref":"HTTPEx.Backend.Mock.Expectation.html#increase_call/1-example"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.int_matcher/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:int_matcher/0"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.keyword_list_matcher/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:keyword_list_matcher/0"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.map_matcher/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:map_matcher/0"},{"type":"function","title":"HTTPEx.Backend.Mock.Expectation.match_expects/2","doc":"Tests if the request mets all defined expectations.\n\nThe `match_expects` function will always return a tuple with a boolean\nand a list of fields that did not match.","ref":"HTTPEx.Backend.Mock.Expectation.html#match_expects/2"},{"type":"function","title":"Examples - HTTPEx.Backend.Mock.Expectation.match_expects/2","doc":"A complex example, mixing different matchers:\n\n    iex> expectation =\n    ...>   %Expectation{}\n    ...>   |> Expectation.set_expect!(:body, fn %Request{} = request -> {String.contains?(request.body, \"OK\"), %{\"var\" => 1}} end)\n    ...>   |> Expectation.set_expect!(:headers, [{\"secret\", \"123\"}])\n    ...>   |> Expectation.set_expect!(:path, Regex.compile!(\"api/(? [^/]+)/path\"))\n    ...>   |> Expectation.set_expect!(:query, %{\"user_id\" => \"1337\"})\n    iex> Expectation.match_expects(\n    ...>   %Request{\n    ...>     url: \"http://www.example.com/api/v1/path/test?token=XYZ&user_id=1337\",\n    ...>     body: \"Payload OK!\",\n    ...>     headers: [{\"app\", \"test\"}, {\"secret\", \"123\"}],\n    ...>     method: :post\n    ...>   },\n    ...>   expectation\n    ...> )\n    {true, [:path, :body, :query, :headers], []}\n    iex> Expectation.match_expects(\n    ...>   %Request{\n    ...>     url: \"http://www.example.com/api/v1/path/test?token=XYZ&user_id=1339\",\n    ...>     body: \"Some error\",\n    ...>     headers: [{\"app\", \"test\"}, {\"geheim\", \"123\"}],\n    ...>     method: :post\n    ...>   },\n    ...>   expectation\n    ...> )\n    {\n      false,\n      [:path],\n      [\n      {:body, true, false}, {:query, %{\"user_id\" => \"1337\"}, %{\"token\" => \"XYZ\", \"user_id\" => \"1339\"}}, {:headers, [{\"secret\", \"123\"}], [{\"app\", \"test\"}, {\"geheim\", \"123\"}]}]}","ref":"HTTPEx.Backend.Mock.Expectation.html#match_expects/2-examples"},{"type":"function","title":"HTTPEx.Backend.Mock.Expectation.match_request/2","doc":"Tests if the request matches all fields.\n\nThe `match_request` function will always return a tuple with a boolean,\nthe fields that matched, and a map with collected variables\nfrom regexes or :func matchers that were executed.\n\nThese vars can be used in responses to replace placeholders in responses.","ref":"HTTPEx.Backend.Mock.Expectation.html#match_request/2"},{"type":"function","title":"Examples - HTTPEx.Backend.Mock.Expectation.match_request/2","doc":"A complex example, mixing different matchers:\n\n    iex> expectation =\n    ...>   %Expectation{}\n    ...>   |> Expectation.set_match!(:host, \"www.example.com\")\n    ...>   |> Expectation.set_match!(:port, 80)\n    ...>   |> Expectation.set_match!(:body, fn %Request{} = request -> {String.contains?(request.body, \"OK\"), %{\"var\" => 1}} end)\n    ...>   |> Expectation.set_match!(:headers, [{\"secret\", \"123\"}])\n    ...>   |> Expectation.set_match!(:path, Regex.compile!(\"api/(? [^/]+)/path\"))\n    ...>   |> Expectation.set_match!(:query, %{\"user_id\" => \"1337\"})\n    ...>   |> Expectation.set_match!(:method, :post)\n    iex> Expectation.match_request(\n    ...>   %Request{\n    ...>     url: \"http://www.example.com/api/v1/path/test?token=XYZ&user_id=1337\",\n    ...>     body: \"Payload OK!\",\n    ...>     headers: [{\"app\", \"test\"}, {\"secret\", \"123\"}],\n    ...>     method: :post\n    ...>   },\n    ...>   expectation\n    ...> )\n    {true, [:method, :headers, :query, :body, :host, :path, :port], [], %{\"api_version\" => \"v1\", \"var\" => 1}}\n    iex> Expectation.match_request(\n    ...>   %Request{\n    ...>     url: \"http://www.example.co/api/v2/path/test?token=XYZ\",\n    ...>     body: \"Payload OK!\",\n    ...>     headers: [{\"app\", \"test\"}, {\"secret\", \"123\"}],\n    ...>     method: :post\n    ...>   },\n    ...>   expectation\n    ...> )\n    {false, [:method, :headers, :body, :path, :port], [:host, :query], %{\"api_version\" => \"v2\", \"var\" => 1}}\n\nA couple of examples using the body formatters.\n\nJSON:\n\n    iex> payload = JSON.encode!(%{username: \"test\"})\n    ...> formatted_payload = JSON.encode!(%{username: \"test\"})\n    iex> expectation =\n    ...>   %Expectation{}\n    ...>   |> Expectation.set_match!(:host, \"www.example.com\")\n    ...>   |> Expectation.set_match!(:port, 80)\n    ...>   |> Expectation.set_match!(:body, {formatted_payload, :json})\n    ...>   |> Expectation.set_match!(:method, :post)\n    iex> Expectation.match_request(\n    ...>   %Request{\n    ...>     url: \"http://www.example.com/api/v1/path/test?token=XYZ&user_id=1337\",\n    ...>     body: payload,\n    ...>     method: :post\n    ...>   },\n    ...>   expectation\n    ...> )\n    {true, [:method, :headers, :query, :body, :host, :path, :port], [], %{}}\n\n  XML:\n\n      iex> payload = \" data \"\n      ...> formatted_payload = \" \\n  data\\n \"\n      iex> expectation =\n      ...>   %Expectation{}\n      ...>   |> Expectation.set_match!(:host, \"www.example.com\")\n      ...>   |> Expectation.set_match!(:port, 80)\n      ...>   |> Expectation.set_match!(:body, {formatted_payload, :xml})\n      ...>   |> Expectation.set_match!(:method, :post)\n      iex> Expectation.match_request(\n      ...>   %Request{\n      ...>     url: \"http://www.example.com/api/v1/path/test?token=XYZ&user_id=1337\",\n      ...>     body: payload,\n      ...>     method: :post\n      ...>   },\n      ...>   expectation\n      ...> )\n      {true, [:method, :headers, :query, :body, :host, :path, :port], [], %{}}","ref":"HTTPEx.Backend.Mock.Expectation.html#match_request/2-examples"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.match_result/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:match_result/0"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.matcher/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:matcher/0"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.matcher_field_type/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:matcher_field_type/0"},{"type":"function","title":"HTTPEx.Backend.Mock.Expectation.new!/1","doc":"Builds an Expectation struct from the given Keyword list","ref":"HTTPEx.Backend.Mock.Expectation.html#new!/1"},{"type":"function","title":"Options - HTTPEx.Backend.Mock.Expectation.new!/1","doc":"- description\n- method\n- endpoint\n- body\n- headers\n- host\n- path\n- port\n- query\n- expect_body\n- expect_headers\n- expect_path\n- expect_query\n- calls\n- stacktrace","ref":"HTTPEx.Backend.Mock.Expectation.html#new!/1-options"},{"type":"function","title":"Examples - HTTPEx.Backend.Mock.Expectation.new!/1","doc":"iex> Expectation.new!(method: :get, endpoint: \"http://www.example.com\", response: %{status: 200, body: \"OK\"}, type: :assert)\n    %Expectation{matchers: %{host: \"www.example.com\", method: :get, path: \"/\", port: 80, body: :any, headers: :any, query: :any}, type: :assertion}","ref":"HTTPEx.Backend.Mock.Expectation.html#new!/1-examples"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.regex_matcher/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:regex_matcher/0"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.response_error/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:response_error/0"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.response_func/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:response_func/0"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.response_map/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:response_map/0"},{"type":"function","title":"HTTPEx.Backend.Mock.Expectation.set_expect!/3","doc":"Sets expects field for given Expectation","ref":"HTTPEx.Backend.Mock.Expectation.html#set_expect!/3"},{"type":"function","title":"Examples - HTTPEx.Backend.Mock.Expectation.set_expect!/3","doc":"iex> Expectation.set_expect!(%Expectation{}, :body, \"ok!\")\n    %Expectation{expects: %{body: \"ok!\", headers: :any, path: :any, query: :any}}\n\n    iex> Expectation.set_expect!(%Expectation{}, :body, nil)\n    ** (ArgumentError) Invalid type used for field expectation `body`. Must be one of: `function()`, `String.t()`, `{String.t(), :json | :xml}`, `RegEx.t()`, `:any`, `{:exact_value, any()}`\n\n    iex> Expectation.set_expect!(%Expectation{}, :unknown, nil)\n    ** (ArgumentError) Unknown field `unknown` for expectation","ref":"HTTPEx.Backend.Mock.Expectation.html#set_expect!/3-examples"},{"type":"function","title":"HTTPEx.Backend.Mock.Expectation.set_match!/3","doc":"Sets matcher field for given Expectation","ref":"HTTPEx.Backend.Mock.Expectation.html#set_match!/3"},{"type":"function","title":"Examples - HTTPEx.Backend.Mock.Expectation.set_match!/3","doc":"iex> Expectation.set_match!(%Expectation{}, :host, \"localhost\")\n    %Expectation{matchers: %{body: :any, headers: :any, host: \"localhost\", method: :any, path: :any, port: :any, query: :any}}\n\n    iex> Expectation.set_match!(%Expectation{}, :body, \"OK!\")\n    %Expectation{matchers: %{body: \"OK!\", headers: :any, host: :any, method: :any, path: :any, port: :any, query: :any}}\n\n    iex> Expectation.set_match!(%Expectation{}, :body, {:exact_value, {:form, \"OK!\"}})\n    %Expectation{matchers: %{body: {:exact_value, {:form, \"OK!\"}}, headers: :any, host: :any, method: :any, path: :any, port: :any, query: :any}}\n\n    iex> Expectation.set_match!(%Expectation{}, :body, {\"{}\", :json})\n    %Expectation{matchers: %{body: {\"{}\", :json}, headers: :any, host: :any, method: :any, path: :any, port: :any, query: :any}}\n\n    iex> Expectation.set_match!(%Expectation{}, :body, {\" a \", :xml})\n    %Expectation{matchers: %{body: {\" a \", :xml}, headers: :any, host: :any, method: :any, path: :any, port: :any, query: :any}}\n\n    iex> Expectation.set_match!(%Expectation{}, :body, {\"foo=bar\", :form})\n    %Expectation{matchers: %{body: {\"foo=bar\", :form}, headers: :any, host: :any, method: :any, path: :any, port: :any, query: :any}}\n\n    iex> Expectation.set_match!(%Expectation{}, :host, nil)\n    ** (ArgumentError) Invalid type used for field matcher `host`. Must be one of: `function()`, `String.t()`, `RegEx.t()`, `:any`\n\n    iex> Expectation.set_match!(%Expectation{}, :unknown, :any)\n    ** (ArgumentError) Unknown field `unknown` for matcher","ref":"HTTPEx.Backend.Mock.Expectation.html#set_match!/3-examples"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.string_formats/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:string_formats/0"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.string_matcher/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:string_matcher/0"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.string_with_format_matcher/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:string_with_format_matcher/0"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.t/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:t/0"},{"type":"function","title":"HTTPEx.Backend.Mock.Expectation.to_client_response/3","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#to_client_response/3"},{"type":"function","title":"HTTPEx.Backend.Mock.Expectation.to_response/3","doc":"Generates a fake response from a map, an error tuple or a http driver struct.\nReplaces any vars if a response map has the option `replace_body_vars` set to `true`.\n\nThe response can also be a function. This function receives a Request.t() and must return\na valid response().","ref":"HTTPEx.Backend.Mock.Expectation.html#to_response/3"},{"type":"function","title":"HTTPEx.Backend.Mock.Expectation.validate_expectations/2","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#validate_expectations/2"},{"type":"function","title":"HTTPEx.Backend.Mock.Expectation.validate_expects_value/2","doc":"Validates if the field with the given value is allowed as an expects","ref":"HTTPEx.Backend.Mock.Expectation.html#validate_expects_value/2"},{"type":"function","title":"HTTPEx.Backend.Mock.Expectation.validate_matcher_value/2","doc":"Validates if the field with the given value is allowed as a matcher","ref":"HTTPEx.Backend.Mock.Expectation.html#validate_matcher_value/2"},{"type":"function","title":"Examples - HTTPEx.Backend.Mock.Expectation.validate_matcher_value/2","doc":"An exact value matcher (note: uses ===):\n\n    iex> Expectation.validate_matcher_value(:body, {:exact_value, {:form, [a: \"data\"]}})\n    :ok\n\nNote, this matcher cannot be used on all fields:\n\n    iex> Expectation.validate_matcher_value(:host, {:exact_value, \"some-host\"})\n    {:error, :invalid_field_type}\n\nA string matcher:\n\n    iex> Expectation.validate_matcher_value(:host, \"localhost\")\n    :ok\n\nYou can also use a function and either return a true/false:\n\n    iex> Expectation.validate_matcher_value(:host, fn _request -> true end)\n    :ok\n\nNote, that the function has to have an arity of one.\nOther function arity's will return an error:\n\n    iex> Expectation.validate_matcher_value(:host, fn -> true end)\n    {:error, :invalid_field_type}\n\nYou can also supply regexes:\n\n    iex> Expectation.validate_matcher_value(:path, Regex.compile!(\"http://localhost:5000/*\"))\n    :ok\n\nA wildcard:\n\n    iex> Expectation.validate_matcher_value(:path, :any)\n    :ok\n\nMatch keyword lists with String.t() values on both sides:\n\n    iex> Expectation.validate_matcher_value(:headers, [{\"Content-Type\", \"application/json\"}])\n    :ok\n\nOr with a Regex.t():\n\n    iex> Expectation.validate_matcher_value(:headers, [{\"Content-Type\", ~r/application/}])\n    :ok\n\nInvalid lists, tuples. types are not valid:\n\n    iex> Expectation.validate_matcher_value(:headers, [{\"Content-Type\", :value}])\n    {:error, :invalid_field_type}\n\n    iex> Expectation.validate_matcher_value(:headers, [])\n    {:error, :invalid_field_type}\n\n    iex> Expectation.validate_matcher_value(:headers, [{\"Content-Type\", \"Value\", \"Other-Value\"}])\n    {:error, :invalid_field_type}\n\n    iex> Expectation.validate_matcher_value(:headers, \"Content-Type: application/json\")\n    {:error, :invalid_field_type}\n\nSome fields require an enum, like `method`:\n\n    iex> Expectation.validate_matcher_value(:method, :post)\n    :ok\n\n    iex> Expectation.validate_matcher_value(:method, :get)\n    :ok\n\n    iex> Expectation.validate_matcher_value(:method, \"get\")\n    {:error, :invalid_field_type}\n\nYou can also match a map. This is useful if you want to match query params:\n\n    iex> Expectation.validate_matcher_value(:query, %{\"user_id\" => \"1234\"})\n    :ok\n\n    iex> Expectation.validate_matcher_value(:query, %{user_id: \"1234\"})\n    {:error, :invalid_field_type}\n\n    iex> Expectation.validate_matcher_value(:query, %{})\n    {:error, :invalid_field_type}","ref":"HTTPEx.Backend.Mock.Expectation.html#validate_matcher_value/2-examples"},{"type":"type","title":"HTTPEx.Backend.Mock.Expectation.wildcard_matcher/0","doc":"","ref":"HTTPEx.Backend.Mock.Expectation.html#t:wildcard_matcher/0"},{"type":"module","title":"HTTPEx.Backend.Mock.Form","doc":"","ref":"HTTPEx.Backend.Mock.Form.html"},{"type":"function","title":"HTTPEx.Backend.Mock.Form.normalize/1","doc":"Parses the given form data and removes any clutter from it, so you can\nsafely compare it with another form data formatted  string.","ref":"HTTPEx.Backend.Mock.Form.html#normalize/1"},{"type":"module","title":"HTTPEx.Backend.Mock.JSON","doc":"","ref":"HTTPEx.Backend.Mock.JSON.html"},{"type":"function","title":"HTTPEx.Backend.Mock.JSON.normalize/1","doc":"Parses the given JSON and removes any clutter from it, so you can\nsafely compare it with another JSON string.","ref":"HTTPEx.Backend.Mock.JSON.html#normalize/1"},{"type":"module","title":"HTTPEx.Backend.Mock.TestHelpers","doc":"This can be used in your test suite for convenience.\nThese macros add tracing to your expectations making them easier to spot if you\nmiss expected calls.","ref":"HTTPEx.Backend.Mock.TestHelpers.html"},{"type":"macro","title":"HTTPEx.Backend.Mock.TestHelpers.assert_no_request!/1","doc":"","ref":"HTTPEx.Backend.Mock.TestHelpers.html#assert_no_request!/1"},{"type":"macro","title":"HTTPEx.Backend.Mock.TestHelpers.expect_request!/1","doc":"","ref":"HTTPEx.Backend.Mock.TestHelpers.html#expect_request!/1"},{"type":"macro","title":"HTTPEx.Backend.Mock.TestHelpers.stub_request!/1","doc":"","ref":"HTTPEx.Backend.Mock.TestHelpers.html#stub_request!/1"},{"type":"module","title":"HTTPEx.Backend.Mock.XML","doc":"","ref":"HTTPEx.Backend.Mock.XML.html"},{"type":"function","title":"HTTPEx.Backend.Mock.XML.#element#/5","doc":"","ref":"HTTPEx.Backend.Mock.XML.html##element#/5"},{"type":"function","title":"HTTPEx.Backend.Mock.XML.#root#/4","doc":"","ref":"HTTPEx.Backend.Mock.XML.html##root#/4"},{"type":"function","title":"HTTPEx.Backend.Mock.XML.#text#/1","doc":"","ref":"HTTPEx.Backend.Mock.XML.html##text#/1"},{"type":"function","title":"HTTPEx.Backend.Mock.XML.#xml-inheritance#/0","doc":"","ref":"HTTPEx.Backend.Mock.XML.html##xml-inheritance#/0"},{"type":"function","title":"HTTPEx.Backend.Mock.XML.normalize/1","doc":"Parses the given XML and removes any clutter from it, so you can\nsafely compare it with another XML string.","ref":"HTTPEx.Backend.Mock.XML.html#normalize/1"},{"type":"module","title":"HTTPEx.Clients","doc":"","ref":"HTTPEx.Clients.html"},{"type":"macro","title":"HTTPEx.Clients.def_request/0","doc":"","ref":"HTTPEx.Clients.html#def_request/0"},{"type":"macro","title":"HTTPEx.Clients.def_request_options/0","doc":"","ref":"HTTPEx.Clients.html#def_request_options/0"},{"type":"macro","title":"HTTPEx.Clients.def_to_client_response/0","doc":"","ref":"HTTPEx.Clients.html#def_to_client_response/0"},{"type":"macro","title":"HTTPEx.Clients.def_to_response/0","doc":"","ref":"HTTPEx.Clients.html#def_to_response/0"},{"type":"module","title":"HTTPEx.Clients.Finch","doc":"Finch driver","ref":"HTTPEx.Clients.Finch.html"},{"type":"function","title":"HTTPEx.Clients.Finch.define_request_functions/0","doc":"Function to generate request functions to execute actual Finch requests","ref":"HTTPEx.Clients.Finch.html#define_request_functions/0"},{"type":"function","title":"HTTPEx.Clients.Finch.define_request_options_functions/0","doc":"Function to generate request_options functions for Finch requests","ref":"HTTPEx.Clients.Finch.html#define_request_options_functions/0"},{"type":"function","title":"HTTPEx.Clients.Finch.define_to_client_response_functions/0","doc":"Function to generate mock responses","ref":"HTTPEx.Clients.Finch.html#define_to_client_response_functions/0"},{"type":"function","title":"HTTPEx.Clients.Finch.define_to_response_functions/0","doc":"Function to generate to_response functions for Finch requests","ref":"HTTPEx.Clients.Finch.html#define_to_response_functions/0"},{"type":"module","title":"HTTPEx.Clients.HTTPoison","doc":"HTTPoison driver","ref":"HTTPEx.Clients.HTTPoison.html"},{"type":"function","title":"HTTPEx.Clients.HTTPoison.define_request_functions/0","doc":"Function to generate request functions to execute actual HTTPoison requests","ref":"HTTPEx.Clients.HTTPoison.html#define_request_functions/0"},{"type":"function","title":"HTTPEx.Clients.HTTPoison.define_request_options_functions/0","doc":"Function to generate request_options functions for HTTPoison requests","ref":"HTTPEx.Clients.HTTPoison.html#define_request_options_functions/0"},{"type":"function","title":"HTTPEx.Clients.HTTPoison.define_to_client_response_functions/0","doc":"Function to generate mock responses","ref":"HTTPEx.Clients.HTTPoison.html#define_to_client_response_functions/0"},{"type":"function","title":"HTTPEx.Clients.HTTPoison.define_to_response_functions/0","doc":"Function to generate to_response functions for HTTPoison requests","ref":"HTTPEx.Clients.HTTPoison.html#define_to_response_functions/0"},{"type":"module","title":"HTTPEx.Logging","doc":"Handles all the HTTP logging and tracing","ref":"HTTPEx.Logging.html"},{"type":"function","title":"HTTPEx.Logging.log/1","doc":"","ref":"HTTPEx.Logging.html#log/1"},{"type":"function","title":"HTTPEx.Logging.span/2","doc":"","ref":"HTTPEx.Logging.html#span/2"},{"type":"function","title":"HTTPEx.Logging.trace/1","doc":"","ref":"HTTPEx.Logging.html#trace/1"},{"type":"extras","title":"HttpEx","doc":"# HttpEx\n\nHttpEx is an Elixir HTTP abstraction library to easily log, trace and mock your HTTP calls.\n\n![Build status](https://github.com/wuunder/http_ex/actions/workflows/ci.yml/badge.svg)\n[![Coverage Status](https://coveralls.io/repos/github/wuunder/http_ex/badge.svg?branch=main)](https://coveralls.io/github/wuunder/http_ex?branch=main)","ref":"readme.html"},{"type":"extras","title":"Installation - HttpEx","doc":"The package [HTTPEx](https://hex.pm/packages/http_ex) can be installed\nby adding `http_ex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:http_ex, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.2 (2025-03-03) - Changelog","doc":"","ref":"changelog.html#v0-2-2025-03-03"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Fixed a bug where the export logging would sometimes trigger twice \n  instead of once.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.1 (2025-03-03) - Changelog","doc":"First release","ref":"changelog.html#v0-1-2025-03-03"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}